"""Return the chi square distance beetween the two given dictionaries."""
from typing import Dict


def chi_square(a: Dict, b: Dict) -> float:
    """Return the chi square distance beetween the two given dictionaries.

    Parameters
    ----------------------------
    a: Dict,
        First dictionary to consider.
    b: Dict,
        Second dictionary to consider.

    Returns
    ----------------------------
    Return the chi square distance beetween the two given dictionaries.
    """

    total = 0
    for k, aval in a.items():
        try:
            bval = b[k]
            total += (aval - bval) * (aval - bval) / (aval + bval)
        except KeyError:
            total += aval

    for k in b:
        if k not in a:
            total += b[k]

    return total



"""Determine the Kullback Leibler divergence beetween the given dictionaries."""
from math import log, inf
from typing import Dict
from .distances_utils import sort


def kullback_leibler(p: Dict, q: Dict) -> float:
    """Determine the Kullback Leibler divergence beetween the given dictionaries.

    Parameters
    ----------------------------
    p: Dict,
        First dictionary to consider.
    q: Dict,
        Second dictionary to consider.

    Returns
    ----------------------------
    Return the Kullback Leibler divergence beetween the given dictionaries.
    """
    total = 0
    overlap = 0
    _, small = sort(p, q)
    for key in small:
        try:
            total += p[key] * log(p[key] / q[key])
            overlap += 1
        except KeyError:
            pass

    if overlap == 0 and (len(p) != 0 or len(q) != 0):
        return inf
    return total
